﻿using HeuristicLab.Clients.Access.Administration;
using HeuristicLab.Clients.Hive.WebJobManager.Services;
using Microsoft.AspNet.SignalR;
using Newtonsoft.Json;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading;
using System.Threading.Tasks;

namespace HeuristicLab.Clients.Hive.WebJobManager.Hubs
{
    public class CalendarHub : Hub
    {
        private WebLoginService weblog;
        private Guid userId;
        private HiveAdminClientWeb adminClient;
        private void loader()
        {
            weblog = WebLoginService.Instance;
            string uid = Context.QueryString["userid"];
            if (uid == null || uid == "" || Guid.Parse(uid) == Guid.Empty)
            {
                userId = Guid.Empty;
            }
            else
            {
                userId = Guid.Parse(uid);
                adminClient = weblog.getAdminClient(userId);
            }
        }
        public void requestInfo()
        {
            loader();
            adminClient.Refresh();
            
            var res = JsonConvert.SerializeObject(adminClient.Resources);
            Clients.Caller.processData(res);
        }
        public void requestDownTime(string id)
        {
            loader();
            adminClient.Refresh();
            Guid t = Guid.Parse(id);
            adminClient.DowntimeForResourceId = t;
            adminClient.RefreshCalendar();
            var down = JsonConvert.SerializeObject(adminClient.Downtimes);
            Clients.Caller.processDowntime(id, down);
        }
        public void saveCalendar(string id, string[] del, string[][] add, string[][] upd, bool fresh, bool last)
        {
            loader();
            var calid = Guid.Parse(id);
            adminClient.Refresh();
            adminClient.DowntimeForResourceId = calid;
            adminClient.RefreshCalendar();
            var downlist = adminClient.Downtimes.ToList();
            foreach(var s in del)
            {
                var gu = Guid.Parse(s);
                var el = downlist.Find(x => x.Id == gu);
                adminClient.Delete(el);
            }
            foreach (var s in add)
            {
                var obj = new Downtime();
                obj.Id = Guid.Empty;
                if (s[0] == "Unavailable")
                    obj.DowntimeType = DowntimeType.Offline;
                else
                    obj.DowntimeType = DowntimeType.Shutdown;
                obj.StartDate = (new DateTime(1970, 1, 1)).AddMilliseconds(double.Parse(s[1]));
                obj.EndDate = (new DateTime(1970, 1, 1)).AddMilliseconds(double.Parse(s[2]));
                if (s[3] == "true")
                    obj.AllDayEvent = true;
                else
                    obj.AllDayEvent = false;
                obj.ResourceId = Guid.Parse(s[4]);
                adminClient.Store(obj, CancellationToken.None);
            }
            foreach(var s in upd)
            {
                 var obj = downlist.Find(x => x.Id == Guid.Parse(s[0]));
                if (s[1] == "Unavailable")
                    obj.DowntimeType = DowntimeType.Offline;
                else
                    obj.DowntimeType = DowntimeType.Shutdown;
                obj.StartDate = (new DateTime(1970, 1, 1)).AddMilliseconds(double.Parse(s[2]));
                obj.EndDate = (new DateTime(1970, 1, 1)).AddMilliseconds(double.Parse(s[3]));
                if (s[4] == "true")
                    obj.AllDayEvent = true;
                else
                    obj.AllDayEvent = false;
                 //adminClient.Store(obj, CancellationToken.None); Throws error
            }
            if( last)
            {
                Clients.Caller.savingAllDone();
            }
            else if (fresh)
            {
                Clients.Caller.savingCurrentDone();
            }
        }
    }
}
